import os
import sys

from dotenv import load_dotenv

load_dotenv()
from language_local.lang_code import LangCode
from message_local.Recipient import Recipient

script_directory = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(script_directory, '..'))
from src.smartlink import (VERIFY_EMAIL_ADDRESS_ACTION_ID,  # noqa: E402
                           SmartLinkLocal)

# TODO: get_test_entity_id
TEST_CAMPAIGN_ID = 1
TEST_SMARTLINK_TYPE_ID = 2

smartlink_local = SmartLinkLocal()
from_recipient = Recipient(user_id=1,
                           contact_id=2,
                           email_address="test1@gmail.com")

to_recipient = Recipient(person_id=1,
                         telephone_number="+972501234567",
                         preferred_language=LangCode.ENGLISH.value,
                         email_address="test2@gmail.com")


def test_insert_and_get():
    smartlink_details = smartlink_local.insert(smartlink_type_id=TEST_SMARTLINK_TYPE_ID,
                                               from_recipient=from_recipient.to_json(),
                                               to_recipient=to_recipient.to_json(),
                                               campaign_id=TEST_CAMPAIGN_ID)
    assert smartlink_details["smartlink_id"] > 0  # no error

    expected_result = smartlink_local.get_smartlink_by_id(smartlink_id=smartlink_details["smartlink_id"])

    assert smartlink_details["smartlink_identifier"] == expected_result["smartlink_identifier"]
    # assert smartlink_details["from_user_id"] == 1
    # assert smartlink_details["from_contact_id"] == 2
    assert smartlink_details["from_email_address_old"] == from_recipient.get_email_address()

    # assert smartlink_details["to_person_id"] == 1
    # assert smartlink_details["to_phone_id"] == to_recipient.get_normizalied_phone()
    assert smartlink_details["lang_code"] == to_recipient.get_preferred_language()

    assert smartlink_details["campaign_id"] == TEST_CAMPAIGN_ID
    assert smartlink_details["action_id"] == VERIFY_EMAIL_ADDRESS_ACTION_ID


def test_execute():
    smartlink_details = smartlink_local.insert(smartlink_type_id=TEST_SMARTLINK_TYPE_ID,
                                               from_recipient=from_recipient.to_json(),
                                               to_recipient=to_recipient.to_json(),
                                               campaign_id=TEST_CAMPAIGN_ID)
    assert smartlink_details["smartlink_id"] > 0  # no error

    smartlink_details = smartlink_local.execute(smartlink_identifier=smartlink_details["smartlink_identifier"])

    smartlink_local.set_schema(schema_name="logger")
    select_clause_value = "return_code"
    execution_details = smartlink_local.select_one_dict_by_id(view_table_name="logger_view",
                                                              select_clause_value=select_clause_value,
                                                              id_column_name="session",
                                                              id_column_value=smartlink_details["session_id"])
    assert execution_details[select_clause_value] == 0



import traceback
from http import HTTPStatus

from lambda_decorators import cors_headers
from sdk.src.utilities import create_http_body

from .smartlink import SmartLinkLocal

smartlink_local = SmartLinkLocal()


@cors_headers
# I'm not sure if we are going to use this handler, let's keep it for the future
def execute_smartlink_handler(event, context):
    # TODO Add optional parameter to logger.start()
    #  which called api_call, so logger.start will call api_management
    #  to insert into api_call_table all fields including session_id.

    # get parameters from event
    smartlink_identifier = event['pathParameters'].get("identifier")

    # TODO Please make sure that inside execute() we SELECT smartlink_view,
    #  if !UserContext.isLogin() UserContext.set_effective_profile_id(profile_id)
    try:
        smartlink_details = smartlink_local.execute(smartlink_identifier=smartlink_identifier)

        body = {
            "message": "Executed SmartLink successfully with smartlink_identifier=" + smartlink_identifier,
            "input": event,
            "smartlink_details": smartlink_details,
        }

        response = {
            "statusCode": HTTPStatus.OK,
            "body": create_http_body(body)
        }

    except Exception as e:
        body = {
            "message": "Failed to execute SmartLink with smartlink_identifier=" + smartlink_identifier,
            "input": event,
            "error": str(e),
            "traceback": traceback.format_exc()
        }

        response = {
            "statusCode": HTTPStatus.INTERNAL_SERVER_ERROR,
            "body": create_http_body(body)
        }
        traceback.print_exc()

    return response


@cors_headers
# Get SmartLink Data by Identifier
def get_smartlink_details_handler(event, context):
    # get parameters from the event
    smartlink_identifier = event['pathParameters'].get("identifier")

    try:
        smartlink_details = smartlink_local.get_smartlink_by_identifier(smartlink_identifier=smartlink_identifier)
        body = {
            "message": "Retrieved SmartLink successfully with smartlink_identifier=" + smartlink_identifier,
            "input": event,
            "smartlink_details": {k: str(v) for k, v in smartlink_details.items()}
        }

        response = {
            "statusCode": HTTPStatus.OK,
            "body": create_http_body(body)
        }

    except Exception as e:
        body = {
            "message": "Failed to retrieve SmartLink with smartlink_identifier=" + smartlink_identifier,
            "input": event,
            "error": str(e),
            "traceback": traceback.format_exc()
        }

        response = {
            "statusCode": HTTPStatus.INTERNAL_SERVER_ERROR,
            "body": create_http_body(body)
        }
        traceback.print_exc()

    return response

# @cors_headers
# Not used currently
# def create_smartlink_handler(event, context):
#     from_recipient = json.loads(event.get("from_recipient"))
#     to_recipient = json.loads(event.get("to_recipient"))
#     campaign_id = event.get("campaign_id")
#     action_id = event.get("action_id")
#
#     # create a SmartLink
#     smartlink_id = smartlink_local.insert(from_recipient=from_recipient, to_recipient=to_recipient,
#                                           campaign_id=campaign_id, action_id=action_id)
#
#     body = {
#         "message": "Created smartlink successfully with smartlink_identifier" + str(smartlink_id),
#         "input": event,
#         "smartlink_id": smartlink_id
#     }
#     response = {
#         "statusCode": HTTPStatus.OK,
#         "body": create_http_body(body)
#     }
#
#     return response
